/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: GetReportsObjectsCommand.as 19356 2010-03-04 21:02:36Z mancelin $
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.admin.auditconsole.command {
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;

import mx.collections.ArrayCollection;
import mx.controls.Alert;
import mx.rpc.IResponder;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;

import org.ow2.jasmine.kerneos.common.event.KerneosNotificationEvent;
import org.ow2.jasmine.kerneos.common.event.ServerSideExceptionEvent;
import org.ow2.jasmine.kerneos.common.view.ServerSideException;
import org.ow2.jonas.admin.auditconsole.business.IAuditConsoleDelegate;
import org.ow2.jonas.admin.auditconsole.model.AuditConsoleModelLocator;
import com.adobe.cairngorm.control.CairngormEventDispatcher;

import org.ow2.jonas.admin.auditconsole.vo.GenericAuditReportVO;
import org.ow2.jonas.admin.auditconsole.vo.InvocationAuditReportVO;
import org.ow2.jonas.admin.auditconsole.vo.JNDIAuditReportVO;
import org.ow2.jonas.admin.auditconsole.vo.JaxwsAuditReportVO;
import org.ow2.jonas.admin.auditconsole.vo.WebInvocationAuditReportVO;

/**
 * Command that call Java side methods.
 */
public class GetReportsObjectsCommand implements ICommand, IResponder {
    /**
    * Send the event to the java side, using the business layer of the pattern
    */
    public function execute( event:CairngormEvent ):void {
        var delegate:IAuditConsoleDelegate = AuditConsoleModelLocator.getInstance().getModuleDelegate()
        delegate.responder = this;
        delegate.getReportObjects();
    }
   /**
    * Get the result of the java side. this method is called on each event from
    * Java.
    */
    public function result( event : Object ):void {
        var model:AuditConsoleModelLocator = AuditConsoleModelLocator.getInstance();
        var reports:ArrayCollection = (event as ResultEvent).result as ArrayCollection;
        if(reports.length != 0){
            model.canObject.dispatchEvent(
                new KerneosNotificationEvent(KerneosNotificationEvent.KERNEOS_NOTIFICATION,
                                             reports.length + " new audit reports received right now",
                                             KerneosNotificationEvent.INFO));
            for each (var report:GenericAuditReportVO in reports){
                if( report is WebInvocationAuditReportVO ) {
                    var targetWeb:String = (report as WebInvocationAuditReportVO).target.replace("webcontainer/", "");
                    targetWeb = targetWeb.replace("root/", "");
                    if (targetWeb.charAt(targetWeb.length) != '/') {
                        targetWeb += "/";
                    }
                    targetWeb += (report as WebInvocationAuditReportVO).businessMethod;
                    model.tomcat.addBean(targetWeb);
                } else if( report is InvocationAuditReportVO ) {
                     var targetEjb:String = (report as InvocationAuditReportVO).target.replace("easybeans/", "");
                     model.easybeans.addBean(targetEjb);
                } else if( report is JNDIAuditReportVO ) {
					 var reportJNDI:JNDIAuditReportVO = (report as JNDIAuditReportVO);
					 var targetJNDI:String = reportJNDI.target + "/" + reportJNDI.businessMethod;
                     model.JNDI.addBean(targetJNDI);
                } else if( report is JaxwsAuditReportVO ) {
					 var jaxws:JaxwsAuditReportVO = (report as JaxwsAuditReportVO);
                     model.jaxws.addBean(jaxws.gridTarget);
                }
                model.addReport(report);
            }
        }
    }
    /**
    * Handle fault messages
    */
     public function fault( info : Object ) : void {
        //Alert.show("Error: "+info);
        var faultEvent : FaultEvent = FaultEvent(info);
        var serverSideExceptionEvent : ServerSideExceptionEvent =
            new ServerSideExceptionEvent(
                "serverSideException",
                new ServerSideException("Error while Executing the action",
                                        "The operation could not be performed."
                                        + "\n" + faultEvent.fault.faultCode
                                        + "\n" + faultEvent.fault.faultString,
                                        faultEvent.fault.getStackTrace()));
        CairngormEventDispatcher.getInstance().dispatchEvent(serverSideExceptionEvent);
    }
}
}
