/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AuditConsoleModelLocator.as 19387 2010-03-09 10:31:40Z mancelin $
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.admin.auditconsole.model {

import com.adobe.cairngorm.model.ModelLocator;

import mx.collections.ArrayCollection;
import mx.collections.XMLListCollection;
import mx.utils.UIDUtil;

import org.ow2.jonas.admin.auditconsole.business.AuditConsoleDelegate;
import org.ow2.jonas.admin.auditconsole.business.IAuditConsoleDelegate;
import org.ow2.jonas.admin.auditconsole.view.MainViewClass;
import org.ow2.jonas.admin.auditconsole.view.NodeVisualGraph;
import org.ow2.jonas.admin.auditconsole.view.tree.EjbTreeBranch;
import org.ow2.jonas.admin.auditconsole.view.tree.JndiTreeBranch;
import org.ow2.jonas.admin.auditconsole.view.tree.JaxwsTreeBranch;
import org.ow2.jonas.admin.auditconsole.view.tree.WebTreeBranch;
import org.ow2.jonas.admin.auditconsole.vo.GenericAuditReportVO;
import org.ow2.jonas.admin.auditconsole.vo.InvocationAuditReportVO;
import org.ow2.jonas.admin.auditconsole.vo.JNDIAuditReportVO;
import org.ow2.jonas.admin.auditconsole.vo.JaxwsAuditReportVO;
import org.ow2.jonas.admin.auditconsole.vo.WebInvocationAuditReportVO;

/**
  * The model locator for the Module.
  */
[Bindable]
public class AuditConsoleModelLocator implements ModelLocator {

    private static const AUDIT_REPORT_LIMIT:int = 2000;

    public var componentID:String = UIDUtil.createUID();

    public var server:XML = <tree><node label="JOnAS" type="JONAS" target="JOnAS" /></tree>;

    public var easybeans:EjbTreeBranch = new EjbTreeBranch("EasyBeans", "/JOnAS/EasyBeans");

    public var tomcat:WebTreeBranch = new WebTreeBranch("Tomcat", "/JOnAS/Tomcat");

    public var JNDI:JndiTreeBranch = new JndiTreeBranch("JNDI", "/JOnAS/JNDI");

    public var jaxws:JaxwsTreeBranch = new JaxwsTreeBranch("Jaxws", "/JOnAS/Jaxws");

    public var config:ConfigurationModel = new ConfigurationModel();

    public var chart:ChartModel;

    public var tops:TopsAuditModel = new TopsAuditModel();

    public var stats:ServerHealthModel = new ServerHealthModel();

    private static var model : AuditConsoleModelLocator = null;

    private var myDelegate : IAuditConsoleDelegate = null;

    private var can:MainViewClass = null;

    private var reportsObj:ArrayCollection = null;

    private var treeTooltipReports:ArrayCollection = null;

    private var filterTooltip:String = "in";

    private var beansObj:ArrayCollection = null;

    private var serverData:XMLListCollection = new XMLListCollection(server.node);

    private var filter:String = "in";

    private var message:String = "";

    private var topdown:Boolean = false;

	private var listeningAudit:Boolean = false;

	/**
	 * Depth of nodes to use when searching nodes
	 */
	public var treeSearchDepth : Number = 2;

	/**
	 * Max results when searching tree nodes
	 */
	public var treeSearchMaxResults : Number = 50;

	/**
	 * KeyWord used for the search.
	 */
	public var treeSearchKeyword : String = ".*";

	/**
	 * Visual Graph component.
	 */
	public var vgraph : NodeVisualGraph = null;

	/**
	 * Showing Graph ?
	 */
	public var showingGraph : Boolean = false;

	/**
	 * Information on all the available root nodes.
	 */
	[ArrayElementType("org.ow2.jonas.admin.auditconsole.vo.NodeInfoVO")]
	public var rootNodesInfo : ArrayCollection;

	/**
     * Display panel chart or details ?
    */
	public var panelChart:Boolean = true;

    function AuditConsoleModelLocator() {
        super();
        this.reportsObj = new ArrayCollection();
        this.beansObj = new ArrayCollection();
        treeTooltipReports = new ArrayCollection();
        this.reportsObj.filterFunction = filterReports;
        treeTooltipReports.filterFunction = treeFilterReports;
        server.node.(@label == "JOnAS")[0].appendChild(easybeans.node);
        server.node.(@label == "JOnAS")[0].appendChild(tomcat.node);
        server.node.(@label == "JOnAS")[0].appendChild(JNDI.node);
        server.node.(@label == "JOnAS")[0].appendChild(jaxws.node);
        serverData = new XMLListCollection(server.node);
        if (model != null) {
            throw new Error("Only one ModelLocator has to be set");
        }

        // Construct the Chart Model
        chart = new ChartModel(this);
    }

    public static function getInstance() : AuditConsoleModelLocator {
        if (AuditConsoleModelLocator.model == null) {
            AuditConsoleModelLocator.model = new AuditConsoleModelLocator();
        }
        return AuditConsoleModelLocator.model;
    }

    public function filterReports(item:Object):Boolean {

        // Quick exit if no filtering has to be applied
        if (this.filter == "in") {
            return true;
        }

        var target:String = "";
        if ((item as GenericAuditReportVO).isWEBInvocation()) {
            if (this.filter == "webinvok") {
                return true;
            }
            target = (item as WebInvocationAuditReportVO).target;
            target += "/" + (item as WebInvocationAuditReportVO).businessMethod;
            if (target.replace("webcontainer/", "").replace("root/","").indexOf(
                    this.filter.replace("/JOnAS/Tomcat", "")) > -1) {
                return true;
            }
        }
        if ((item as GenericAuditReportVO).isEJBInvocation()) {
            if (this.filter == "ejbinvok") {
                return true;
            }
            target = (item as InvocationAuditReportVO).target;
            if (target.replace("easybeans/", "").indexOf(
                    this.filter.replace("/JOnAS/EasyBeans", "")) > -1) {
                return true;
            }
        }
        if ((item as GenericAuditReportVO).isJNDIInvocation()) {
            if (this.filter == "jndiinvok") {
                return true;
            }
            var reportJNDI:JNDIAuditReportVO = (item as JNDIAuditReportVO);
            var targetReportJNDI:String = reportJNDI.target + "/" + reportJNDI.businessMethod;
            var tmpString:String = this.filter.replace("/JOnAS/JNDI/", "");
            if (targetReportJNDI.indexOf(tmpString) > -1) {
                return true;
            }
        }
        if ((item as GenericAuditReportVO).isJaxwsInvocation()) {
            if (this.filter == "jaxwsinvok") {
                // Accept all JAX-WS reports
                return true;
            }

            // else, keep it only if matching the target filter
            var report:JaxwsAuditReportVO = (item as JaxwsAuditReportVO);
            if (report.gridTarget.indexOf(filter) > -1) {
                return true;
            }
        }
        return false;
    }

    public function treeFilterReports(item:Object):Boolean {

        // Quick exit if no filtering has to be applied
        if (this.filterTooltip == "in") {
            return true;
        }

        var target:String = "";
        if ((item as GenericAuditReportVO).isWEBInvocation()) {
            if (this.filterTooltip == "webinvok") {
                return true;
            }
            target = (item as WebInvocationAuditReportVO).target;
            target += "/" + (item as WebInvocationAuditReportVO).businessMethod;
            if (target.replace("webcontainer/", "").replace("root/","").indexOf(
                    this.filterTooltip.replace("/JOnAS/Tomcat", "")) > -1) {
                return true;
            }
        }
        if ((item as GenericAuditReportVO).isEJBInvocation()) {
            if (this.filterTooltip == "ejbinvok") {
                return true;
            }
            target = (item as InvocationAuditReportVO).target;
            if (target.replace("easybeans/", "").indexOf(
                    this.filterTooltip.replace("/JOnAS/EasyBeans", "")) > -1) {
                return true;
            }
        }
        if ((item as GenericAuditReportVO).isJNDIInvocation()) {
            if (this.filterTooltip == "jndiinvok") {
                return true;
            }
			var reportJNDI:JNDIAuditReportVO = (item as JNDIAuditReportVO);
			var targetReportJNDI:String = reportJNDI.target + "/" + reportJNDI.businessMethod;
		    var tmpString:String = this.filterTooltip.replace("/JOnAS/JNDI/", "");
            if (targetReportJNDI.indexOf(tmpString) > -1) {
                return true;
            }
        }
        if ((item as GenericAuditReportVO).isJaxwsInvocation()) {
            if (this.filterTooltip == "jaxwsinvok") {
                // Accept all JAX-WS reports
                return true;
            }

            // else, keep it only if matching the target filter
            var report:JaxwsAuditReportVO = (item as JaxwsAuditReportVO);
            if (report.gridTarget.indexOf(filterTooltip) > -1) {
                return true;
            }
        }
        return false;
    }

    public function addReport(report:GenericAuditReportVO):void {
        if (reportsObj.length >= AUDIT_REPORT_LIMIT) {
            if (topdown) {
                reportsObj.removeItemAt(0);
            } else {
                reportsObj.removeItemAt(reportsObj.length - 1);
            }
        }
        if (topdown) {
            reportsObj.addItem(report);
            treeTooltipReports.addItem(report);
        } else {
            reportsObj.addItemAt(report, 0);
            treeTooltipReports.addItemAt(report, 0);
        }
        chart.addReport(report);
    }

    public function getTreeTooltipMessage(filter:String, node:XML, tooltipFilter:String):String {
        var message:String = "";
        var name:String = node.parent().@label + "/" + filter;
        var totalTime:Number = 0;
        filterTooltip = tooltipFilter;
        treeTooltipReports.refresh();
        for each (var report:GenericAuditReportVO in treeTooltipReports) {
            totalTime += report.duration;
        }
        var meanTime:Number = 0;
        if (treeTooltipReports.length != 0) {
        	meanTime = totalTime / treeTooltipReports.length;
        }
        message = "Data on : " + name + "\r\r"
                + "The general invocations average duration is  " + meanTime.toFixed(3) + " ms.\r"
                + "The total invocations duration is  " + totalTime.toFixed(3) + " ms.\r"
                + "The total number of invocations is  " + treeTooltipReports.length + ".\r"
                ;
        return message;
    }

    public function getModuleDelegate() : IAuditConsoleDelegate {
        if (this.myDelegate == null) {
            this.myDelegate = new AuditConsoleDelegate();
        }
        return this.myDelegate;
    }

    public function set reports(reportsArray:ArrayCollection):void {
        for each (var report:Object in reportsArray) {
            this.reportsObj.addItem(report);
        }
    }

    public function set filterString(filter:String):void {
        this.filter = filter;
        this.reportsObj.refresh();
    }

    public function get reports():ArrayCollection {
        return this.reportsObj;
    }

    public function get serverDatas():XMLListCollection {
        return this.serverData;
    }

    public function get topMessage():String {
        return this.message;
    }

    public function set topMessage(mess:String):void {
        this.message = mess;
    }

    public function get beans():ArrayCollection {
        return this.beansObj;
    }

    public function get topDown():Boolean {
        return topdown;
    }

    public function set topDown(value:Boolean):void {
        topdown = value;
    }

    public function get canObject():MainViewClass {
        return can;
    }

    public function set canObject(value:MainViewClass):void {
        can = value;
    }

	public function get listening():Boolean {
		return listeningAudit;
	}

	public function set listening(value:Boolean):void {
		listeningAudit = value;
	}


}
}
