/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2011 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JNDIAuditReportVO.as 19356 2010-03-04 21:02:36Z mancelin $
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.admin.auditconsole.model {
import mx.collections.ArrayCollection;

import org.ow2.jonas.admin.auditconsole.vo.GenericAuditReportVO;

/**
 * Sliding window for the main chart of the app.
 */
[Bindable]
public class ChartModel {

    public var reportsEJB:ArrayCollection = null;
    public var reportsJNDI:ArrayCollection = null;
    public var reportsWEB:ArrayCollection = null;
    public var reportsJaxws:ArrayCollection = null;
    public var reportsDates:ArrayCollection = null;

    private var model:AuditConsoleModelLocator;

    public function ChartModel(model:AuditConsoleModelLocator) {
        reportsEJB = new ArrayCollection();
        reportsJNDI = new ArrayCollection();
        reportsWEB = new ArrayCollection();
        reportsJaxws = new ArrayCollection();
        reportsDates = new ArrayCollection();

        this.model = model;
    }

    public function addReport(report:GenericAuditReportVO):void {
        if (model.filterReports(report)) {
            if (reportsEJB.length > model.config.auditSamples) {
                reportsEJB.removeItemAt(0);
                reportsWEB.removeItemAt(0);
                reportsJNDI.removeItemAt(0);
                reportsJaxws.removeItemAt(0);
                reportsDates.removeItemAt(0);
            }
            if (report.isEJBInvocation()) {
                reportsEJB.addItem(report);
                reportsWEB.addItem(null);
                reportsJNDI.addItem(null);
                reportsJaxws.addItem(null);
            }
            if (report.isWEBInvocation()) {
                reportsEJB.addItem(null);
                reportsWEB.addItem(report);
                reportsJNDI.addItem(null);
                reportsJaxws.addItem(null);
            }
            if (report.isJNDIInvocation()) {
                reportsEJB.addItem(null);
                reportsWEB.addItem(null);
                reportsJNDI.addItem(report);
                reportsJaxws.addItem(null);
            }
            if (report.isJaxwsInvocation()) {
                reportsEJB.addItem(null);
                reportsWEB.addItem(null);
                reportsJNDI.addItem(null);
                reportsJaxws.addItem(report);
            }
            reportsDates.addItem(report.chartDate);
        }
    }

    public function removeAllReports():void {
        reportsEJB.removeAll();
        reportsJNDI.removeAll();
        reportsWEB.removeAll();
        reportsJaxws.removeAll();
        reportsDates.removeAll();
    }

    public function reinitializeReportCollection():void {
        removeAllReports();
        for (var i:int = model.config.auditSamples; i > -1 ; i--) {
            if (i <= (model.reports.length - 1)) {
                addReport(model.reports.getItemAt(i) as GenericAuditReportVO);
            }
        }
    }
}
}