/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: IconsUtils.as 19387 2010-03-09 10:31:40Z mancelin $
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.admin.auditconsole.util {

/**
 * Parser utility for Jaxws Identifier string returned by the server.
 */
public class JaxwsIdentifierParser {

    public function JaxwsIdentifierParser() {}

    private var _serviceQName:QName;
    private var _portQName:QName;
    private var _operationQName:QName;

    public function parse(identifier:String):void {

        // The expected format is the following:
        // Service / Port / Operation
        //
        // Pseudo grammar:
        // identifier ::= qname '/' qname '/' qname
        // qname ::= '{' namespace '}' localpart
        //
        // Example:
        // {http://jonas.ow2.org/tutorial/jaxws/quote}QuoteReporterService/{http://jonas.ow2.org/tutorial/jaxws/quote}QuoteReporterPort/{http://jonas.ow2.org/tutorial/jaxws/quote}getQuote


        var qnameState:String = "";
        var globalState:String = "SERVICE";
        var index:Number = 0;
        var buffer:String = "";
        var uri:String = "";
        while (index < identifier.length) {
            var current:String = identifier.charAt(index);

            if ("{" == current) {
                qnameState = "URI";
            } else if ("}" == current) {
                qnameState = "LOCAL";
                uri = buffer;
                buffer = "";
            } else {

                if ("LOCAL" == qnameState) {
                    // Parsing a localName
                    if ("/" == current) {

                        // '/' is the end delimiter for QName local part

                        if ("SERVICE" == globalState) {
                            // We've parsed a Service localName
                            _serviceQName = new QName(uri, buffer);
                            globalState = "PORT";
                        } else if ("PORT" == globalState) {
                            // We've parsed a Port localName
                            _portQName = new QName(uri, buffer);
                            globalState = "OPERATION";
                        } else {
                            // We've parsed a Port localName
                            _operationQName = new QName(uri, buffer);
                        }
                        buffer = "";
                    } else if (":" == current) {
                        // We've parsed a Port localName
                        _portQName = new QName(uri, buffer);
                        buffer = "";

                    } else {
                        // Store current char in the buffer
                        buffer += current;
                    }
                } else {
                    // Store current char in the buffer
                    buffer += current;
                }
            }

            // Move to the next char
            index++;
        }

        _operationQName = new QName(uri, buffer);
    }


    public function get serviceQName():QName {
        return _serviceQName;
    }

    public function get portQName():QName {
        return _portQName;
    }

    public function get operationQName():QName {
        return _operationQName;
    }
}
}
