/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.admin.auditconsole.view {
import flash.geom.Point;

import mx.core.ClassFactory;

import org.ow2.jonas.admin.auditconsole.model.AuditConsoleModelLocator;
import org.ow2.jonas.admin.auditconsole.util.IconsUtils;
import org.un.cava.birdeye.ravis.assets.icons.EmbeddedIcons;
import org.un.cava.birdeye.ravis.components.renderers.edgeLabels.BaseEdgeLabelRenderer;
import org.un.cava.birdeye.ravis.graphLayout.data.Graph;
import org.un.cava.birdeye.ravis.graphLayout.layout.HierarchicalLayouter;
import org.un.cava.birdeye.ravis.graphLayout.visual.VisualGraph;
import org.un.cava.birdeye.ravis.graphLayout.visual.edgeRenderers.FlowEdgeRenderer;

public class NodeVisualGraph extends VisualGraph {

    private var model : AuditConsoleModelLocator = AuditConsoleModelLocator.getInstance();

    [Bindable]
    [Embed("/../assets/ui/bean.png")]
    public var easyBeansIcon : Class;

    [Bindable]
    [Embed("/../assets/ui/bean.png")]
    public var localBeanIcon : Class;

    [Bindable]
    [Embed("/../assets/ui/bean-remote.png")]
    public var remoteBeanIcon : Class;

    [Bindable]
    [Embed("/../assets/ui/tomcat.png")]
    public var tomcatIcon : Class;

    [Bindable]
    [Embed("/../assets/ui/jaxws.png")]
    public var jaxwsIcon : Class;


    public function NodeVisualGraph() {
    }


    public function fillNewData(xml : XML) : void {
        model.vgraph.graph = new Graph("XMLAsDocsGraph", false, xml);


        // Select the first available node as starting root node
        if (xml.elements("Node").length() > 0) {
            this.currentRootVNode = this.graph.nodeByStringId(xml.elements("Node")[0].@id).vnode;
			model.vgraph.centerNodeByStringId(xml.elements("Node")[0].@id);
			this._scale = 1;
			this._origin = new Point(0,0);
        } else {
            model.showingGraph = false;
            return;
        }

        // layout (hierarchical)
        var layouter : HierarchicalLayouter = new HierarchicalLayouter(this);
        layouter.orientation = HierarchicalLayouter.ORIENT_TOP_DOWN;
        this.layouter = layouter;

        // Node renderer
        this.itemRenderer = new ClassFactory(AuditIconRenderer);

        // Change the set of icons
        EmbeddedIcons.n02Icon = tomcatIcon;
        EmbeddedIcons.n10Icon = easyBeansIcon;
        EmbeddedIcons.n11Icon = localBeanIcon;
        EmbeddedIcons.n12Icon = remoteBeanIcon;
        EmbeddedIcons.n30Icon = jaxwsIcon;


        // Edge renderer
        this.edgeRenderer = new FlowEdgeRenderer(this.edgeDrawGraphics);
        // Edge label renderer
        this.edgeLabelRenderer = new ClassFactory(BaseEdgeLabelRenderer);

        // Settings
        this.layouter.autoFitEnabled = true;
        this.layouter.disableAnimation = false;

        // distance
        this.maxVisibleDistance = 10;

        // refresh the graph
        this.refresh();
        this.draw();
    }
}
}