/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.admin.auditconsole.view {
import flash.events.Event;
import flash.geom.Point;

import mx.charts.ChartItem;
import mx.charts.ColumnChart;
import mx.charts.HitData;
import mx.charts.series.LineSeries;
import mx.containers.HBox;
import mx.containers.VBox;
import mx.controls.Button;
import mx.controls.CheckBox;
import mx.controls.ColorPicker;
import mx.controls.TextArea;
import mx.controls.Tree;
import mx.events.ColorPickerEvent;
import mx.events.ListEvent;
import mx.events.ToolTipEvent;
import mx.graphics.SolidColor;
import mx.graphics.Stroke;
import mx.managers.ToolTipManager;

import org.ow2.jonas.admin.auditconsole.model.AuditConsoleModelLocator;
import org.ow2.jonas.admin.auditconsole.util.IconsUtils;
import org.ow2.jonas.admin.auditconsole.vo.JaxwsAuditReportVO;


public class ServerReportsClass extends VBox {
    /**
     * Model used for this view.
     */
    [Bindable]
    public var model : AuditConsoleModelLocator = AuditConsoleModelLocator.getInstance();

    [Bindable]
    public var timerCheckBox : CheckBox = null;

    [Bindable]
    public var startbutton : Button = null;

    [Bindable]
    public var pausebutton : Button = null;

    [Bindable]
    public var beanTree : Tree = null;

    [Bindable]
    public var minReport : TextArea = null;

    [Bindable]
    public var myChart : ColumnChart = null;

    [Bindable]
    public var s2 : Stroke = null;

    [Bindable]
    public var sc2 : SolidColor = null;

    [Bindable]
    public var colorPickerEJB : ColorPicker = null;

    [Bindable]
    public var colorPickerWEB : ColorPicker = null;

    [Bindable]
    public var colorPickerJNDI : ColorPicker = null;

    [Bindable]
    public var colorPickerJaxws : ColorPicker = null;



    private var coll : int = 0;

    [Bindable]
    public var selected : String = "JOnAS";

    [Bindable]
    public var selectedNode : XML;

    [Bindable]
    public var selectedInterval : Object;

    [Bindable]
    public var showedLineEJB : LineSeries;

    [Bindable]
    public var showedLineWEB : LineSeries;

    [Bindable]
    public var showedLineJNDI : LineSeries;

    [Bindable]
    public var showedLineJaxws : LineSeries;

    [Bindable]
    public var chartBox : HBox;


    public function ServerReportsClass() {
    }


    protected function initTree() : void {
        for (var i : int = 0; i < beanTree.dataProvider.length; i++) {
            beanTree.expandChildrenOf(beanTree.dataProvider[i], true);
        }
    }

    protected function treeLabel(item : Object) : String {
        var node : XML = item as XML;
        return node.@label;
    }

    protected function treeIcon(item : Object) : Class {
        var node:XML = item as XML;
        return IconsUtils.getInstance().treeIcon(node.@type);
    }


    protected function createCustomTip(event : ToolTipEvent) : void {

        // Replace the ToolTip with our own ToolTip
        // So that we have a Panel title on our ToolTip
        var tooltip : DetailTooltip = new DetailTooltip();
        tooltip.text = event.target.toolTip;
        tooltip.title = "Info";
        event.toolTip = tooltip;
    }

    protected function collapseTree() : void {
        var i : int = 0;
        if (coll == 1) {
            for (i = 0; i < beanTree.dataProvider.length; i++) {
                beanTree.expandChildrenOf(beanTree.dataProvider[i], false);
            }
            coll = 2;
        } else {
            for (i = 0; i < beanTree.dataProvider.length; i++) {
                beanTree.expandChildrenOf(beanTree.dataProvider[i], true);
            }
            coll = 1;
        }
    }


    protected function switchPanels() : void {
        model.panelChart = !model.panelChart;
    }

    protected function chartDataTipFunction(data : HitData) : String {

        // ChartItem.item is the report object
        var cItem:ChartItem = data.chartItem;
        var name:String = "";

        if (cItem.item.type == "Jaxws") {
            // Splitting on '/' won't work with Jaxws reports
            var report:JaxwsAuditReportVO = cItem.item as JaxwsAuditReportVO;
            name = report.gridTarget;
        } else {
            // Usual logic for all other reports type
            var parts:Array = cItem.item.target.split("/");
            name = parts[parts.length - 2] + "/" + parts[parts.length - 1];
            if (cItem.item.type == "WEB") {
                name += "/" + cItem.item.businessMethod;
            }
        }

        // Build the tooltip
        var tooltip:String = "<b>" + model.config.dataAre + "</b><br />" +
                             "Call on <i>" + name + "</i><br />";

        if (cItem.item.type == "WEB") {
            tooltip += "(" + parts[2] + ")<br />";
        }
        tooltip += "Request duration : " + cItem.item.duration + " ms <br />" +
                "Free Memory Before : " + cItem.item.freeMemoryBefore + " bytes <br />" +
                "Free Memory After : " + cItem.item.freeMemoryAfter + " bytes <br />" +
                "GC sweep/mark time : " + cItem.item.sweepMarkTime + " ms <br />" +
                "GC scavenge time : " + cItem.item.scavengeTime + " ms <br />";
        //+ "Parameters Size : " + cItem.item.paramSize + " bytes ";
        return tooltip;
    }

    protected function treeChanged(event : Event) : void {
        selectedNode = Tree(event.target).selectedItem as XML;
        selected = selectedNode.attribute("label")[0];
        if (selected == "JOnAS") {
            model.filterString = "in";
        } else if (selected == "EasyBeans") {
            model.filterString = "ejbinvok";
        } else if (selected == "Tomcat") {
            model.filterString = "webinvok";
        } else if (selected == "JNDI") {
            model.filterString = "jndiinvok";
        } else if (selected == "Jaxws") {
            model.filterString = "jaxwsinvok";
        } else {
            selected = selectedNode.attribute("target")[0];
            model.filterString = selected;
        }
    }


    protected function itemRollOver(event : ListEvent) : void {
        var item : XML = event.itemRenderer.data as XML;
        var selec : String = "";
        if (item != null) {
            selec = item.@label;
            var tooltipFilter : String = "in";
            if (selec == "JOnAS") {
                tooltipFilter = "in";
            } else if (selec == "EasyBeans") {
                tooltipFilter = "ejbinvok";
            } else if (selec == "Tomcat") {
                tooltipFilter = "webinvok";
            } else if (selec == "JNDI") {
                tooltipFilter = "jndiinvok";
            } else if (selec == "Jaxws") {
                tooltipFilter = "jaxwsinvok";
            } else {
                tooltipFilter = item.@target;
            }
            event.target.toolTip = model.getTreeTooltipMessage(selec, item, tooltipFilter);
            var itemX : int = event.itemRenderer.x;
            var itemY : int = event.itemRenderer.y;
            var point : Point = event.currentTarget.contentToGlobal(new Point(itemX, itemY));
            var posX : int = point.x + event.itemRenderer.width + 2;
            var posY : int = point.y;
            if (ToolTipManager.currentToolTip != null) {
                ToolTipManager.currentToolTip.move(posX, posY);
            }
        }
    }

    protected function itemRollOut(event : ListEvent) : void {
        if (event.target.toolTip != null) {
            event.target.toolTip = null;
        }
    }

    protected function changeGraphColorEJB(e : ColorPickerEvent) : void {
        model.config.chartEJBColor = colorPickerEJB.selectedColor;
        var existingStroke : Stroke = showedLineEJB.getStyle("lineStroke");
        var newStroke : Stroke = new Stroke(colorPickerEJB.selectedColor, existingStroke.weight, existingStroke.alpha);
        showedLineEJB.setStyle("lineStroke", newStroke);
    }

    protected function changeGraphColorWEB(e : ColorPickerEvent) : void {
        model.config.chartWEBColor = colorPickerWEB.selectedColor;
        var existingStroke : Stroke = showedLineWEB.getStyle("lineStroke");
        var newStroke : Stroke = new Stroke(colorPickerWEB.selectedColor, existingStroke.weight, existingStroke.alpha);
        showedLineWEB.setStyle("lineStroke", newStroke);
    }

    protected function changeGraphColorJNDI(e : ColorPickerEvent) : void {
        model.config.chartJNDIColor = colorPickerJNDI.selectedColor;
        var existingStroke : Stroke = showedLineJNDI.getStyle("lineStroke");
        var newStroke : Stroke = new Stroke(colorPickerJNDI.selectedColor, existingStroke.weight, existingStroke.alpha);
        showedLineJNDI.setStyle("lineStroke", newStroke);
    }

    protected function changeGraphColorJaxws(e : ColorPickerEvent) : void {
        model.config.chartJaxwsColor = e.color;
        var existingStroke : Stroke = showedLineJaxws.getStyle("lineStroke");
        var newStroke : Stroke = new Stroke(colorPickerJaxws.selectedColor, existingStroke.weight, existingStroke.alpha);
        showedLineJaxws.setStyle("lineStroke", newStroke);
    }
}
}